/*T
   Concepts: KSP^solving a system of linear equations
   Concepts: KSP^Laplacian, 2d
   Processors: n
T*/

/*
Added at the request of Marc Garbey.

Inhomogeneous Laplacian in 2D. Modeled by the partial differential equation

   div \rho grad u = f,  0 < x,y < 1,

   with forcing function

   f = e^{-x^2/\nu} e^{-y^2/\nu}

   with Dirichlet boundary conditions

   u = f(x,y) for x = 0, x = 1, y = 0, y = 1

   or pure Neumman boundary conditions

   This uses multigrid to solve the linear system
 */

static char help[] = "Solves 2D inhomogeneous Laplacian using multigrid.\n\n";

#include <petscdmda.h>
#include <petscksp.h>
#include <petscpcmg.h>
#include "ComputeFunctions.h"
#include "GlobalVariables.h"

extern PetscErrorCode Initialization(DM, int);
extern PetscErrorCode CleanUpWorks();
extern PetscErrorCode CommUpdate();


#undef __FUNCT__
#define __FUNCT__ "main"
int main(int argc,char **argv) {
  int time = 0;
  DM             da;
  PetscReal      norm;
  PetscErrorCode ierr;
  KSP	ksp;
  Vec	x,b,r;
  Mat	A;

  PetscInitialize(&argc,&argv,(char *)0,help);
  MPI_Comm_rank(PETSC_COMM_WORLD, &rank);
  MPI_Comm_size(PETSC_COMM_WORLD, &NumRank);
  printf("NumRank = %d \n", NumRank);


  ierr = KSPCreate(PETSC_COMM_WORLD,&ksp);CHKERRQ(ierr);
  ierr = DMDACreate3d(PETSC_COMM_WORLD,DMDA_BOUNDARY_NONE,DMDA_BOUNDARY_NONE,DMDA_BOUNDARY_NONE,DMDA_STENCIL_STAR,-imax,-jmax,-kmax,1,2,1,1,1,0,0,0,&da);CHKERRQ(ierr);
  ierr = DMDASetUniformCoordinates(da, 0., (imax-1)*dxValue, 0., (jmax-1)*dyValue, 0, (kmax-1)*dzValue);CHKERRQ(ierr);

  ierr = Initialization(da, time);
  ierr = KSPDestroy(&ksp);CHKERRQ(ierr);
  ierr = DMDestroy(&da);CHKERRQ(ierr);

  while(time < timestpNum) {

    printf("Rank = %d is ready to communicate!!\n", rank);
    ierr = CommUpdate();
    MPI_Barrier(PETSC_COMM_WORLD);

    printf("Rank = %d, Computation for time step %d is done!! %d time steps left\n\n", rank, time, (int)(timestpNum-time-1));
    time++; 

  }

  MPI_Barrier(PETSC_COMM_WORLD);
  ierr = CleanUpWorks();
  ierr = PetscFinalize();

  return 0;
}

